"""
    A Report plugin to send a report to bugzilla.redhat.com.
    Copyright (C) 2009 Red Hat, Inc

    Author: Gavin Romig-Koch <gavin@redhat.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import os

import gettext
_ = lambda x: gettext.ldgettext("report", x)

label="bugzilla.redhat.com"
description="Send report to bugzilla.redhat.com"

def report(signature, io):

    attachedFileName = None
    attachedFileDescription = None

    if signature.has_key("pythonUnhandledException"):
        attachedFileName = signature["pythonUnhandledException"].asFileName()
        attachedFileDescription = "Attached traceback automatically from %s." % signature["component"].asString()
    
    return sendToBugzilla(signature["component"].asString(),
                          signature["hashmarkername"].asString(),
                          signature["localhash"].asString(),
                          signature["summary"].asString(),
                          signature["description"].asString(),
                          attachedFileName,
                          attachedFileDescription,
                          io,
                          None)



import filer

def sendToBugzilla( component, hashmarkername, hashvalue, summary, firstComment, fileName, fileDescription, io, bzfiler):

    import rpmUtils.arch
    bugzillaUrl = "https://bugzilla.redhat.com/show_bug.cgi?id=" 

    class BugzillaCommunicationException (Exception):
        pass

    def withBugzillaDo(bz, fn):
        try:
            retval = fn(bz)
            return retval
        except filer.CommunicationError, e:
            msg = _("Your bug could not be filed due to the following error "
                    "when communicating with bugzilla:\n\n%s" % str(e))
            io.failMessage(_("Unable To File Bug"), msg)
            raise BugzillaCommunicationException()

        except (TypeError, ValueError), e:
            msg = _("Your bug could not be filed due to bad information in "
                    "the bug fields.  This is most likely an error in "
                    "the bug filing program:\n\n%s" % str(e))
            io.failMessage(_("Unable To File Bug"), msg)
            raise BugzillaCommunicationException()


    try:
        if not bzfiler:
            bzfiler = filer.BugzillaFiler("https://bugzilla.redhat.com/xmlrpc.cgi",
                                  "http://bugzilla.redhat.com",
                                  filer.getVersion(), filer.getProduct())

        if not bzfiler or not bzfiler.supportsFiling() or not bzfiler.bugUrl:
            io.failMessage(_("Bug Filing Not Supported"),
                           _("Your distribution does not provide a "
                             "supported bug filing system, so you "
                             "cannot save your exception this way."))
            return False

        loginResult = io.queryLogin("bugzilla.redhat.com")
        if loginResult:
            password = loginResult['password']
            username = loginResult['username']
        else:
            io.failMessage(_("No Login Information"),
                _("Please provide a valid username and password."))
            return False
    
        try:
            withBugzillaDo(bzfiler, lambda b: b.login(username, password))
        except filer.LoginError:
            io.failMessage(_("Unable To Login"),
                _("There was an error logging into %s "
                  "using the provided username and "
                  "password.") % bzfiler.displayUrl)
            return False
    
        io.updateLogin("bugzilla.redhat.com",loginResult)

        # FIXME just for testing
        # component = "fastback"
        # hashmarkername = "fastback99"
        # FIXME
    
    
        # Are there any existing bugs with this hash value?  If so we will just
        # add this traceback to the bug report and put the reporter on the CC
        # list.  Otherwise, we need to create a new bug.
        wb = "%s_trace_hash:%s" % (hashmarkername, hashvalue)
        buglist = withBugzillaDo(bzfiler, lambda b: b.query({'status_whiteboard': wb,
                                                           'status_whiteboard_type':'allwordssubstr',
                                                           'bug_status': []}))
        if len(buglist) == 0:
            bug = withBugzillaDo(bzfiler, lambda b: b.createbug(product=bzfiler.getproduct(),
                                           component=component,
                                           version=bzfiler.getversion(),
                                           platform=rpmUtils.arch.getBaseArch(),
                                           bug_severity="medium",
                                           priority="medium",
                                           op_sys="Linux",
                                           bug_file_loc="http://",
                                           summary=summary,
                                           comment=firstComment,
                                           status_whiteboard=wb))
    
            if fileName:
                if not fileDescription:
                    fileDescription = ""
                withBugzillaDo(bug, lambda b: b.attachfile(fileName, fileDescription,
                                     contenttype="text/plain", 
                                     filename=os.path.basename(fileName)))
    
            # Tell the user we created a new bug for them and that they should
            # go add a descriptive comment.
            if bzfiler.displayUrl.endswith("/"):
                bugUrl = "%s%s" % (bzfiler.displayUrl, bug.id())
            else:
                bugUrl = "%s/%s" % (bzfiler.displayUrl, bug.id())
    
            io.successMessage(_("Bug Created"),
                _("A new bug has been created with your traceback attached. "
                  "Please add additional information such as what you were doing "
                  "when you encountered the bug, screenshots, and whatever else "
                  "is appropriate to the following bug:"),
                              bugzillaUrl + str(bug.id()),
                              bugUrl)
            return True
        else:
            bug = buglist[0]
            if fileName:
                if not fileDescription:
                    fileDescription = ""

                withBugzillaDo(bug, lambda b: b.attachfile(fileName,fileDescription,
                                    contenttype="text/plain", 
                                    filename=os.path.basename(fileName)))
            withBugzillaDo(bug, lambda b: b.addCC(username))
    
            # Tell the user which bug they've been CC'd on and that they should
            # go add a descriptive comment.
            if bzfiler.displayUrl.endswith("/"):
                bugUrl = "%s%s" % (bzfiler.displayUrl, bug.id())
            else:
                bugUrl = "%s/%s" % (bzfiler.displayUrl, bug.id())
    
            io.successMessage(_("Bug Updated"),
                _("A bug with your information already exists.  Your account has "
                  "been added to the CC list and your traceback added as a "
                  "comment.  Please add additional descriptive information to the "
                  "following bug:\n\n"),
                              bugzillaUrl + str(bug.id()),
                              bugUrl)

            return True

    except BugzillaCommunicationException:
        # this indicates that doWithBugzilla caught some problem
        # communicating with bugzilla and displayed a message about it
        # and all we want to do now is get out of sendToBugzilla
        return False


